<?php

declare(strict_types=1);

namespace Ig\IgGeodata\Domain\Repository;

use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Persistence\Generic\Mapper\DataMapper;
use TYPO3\CMS\Extbase\Persistence\Generic\Typo3QuerySettings;

/**
 * The repository for Plzs
 */
class PlzRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
{

    public function initializeObject()
    {
        $querySettings = GeneralUtility::makeInstance(Typo3QuerySettings::class);
        $querySettings->setRespectStoragePage(false);
        $this->setDefaultQuerySettings($querySettings);
    }

    public function findNearestPlz(int $plz)
    {
        if ($plz == 0) {
            return false;
        }
        
        $entry = $this->findOneByPlz($plz);
        if($entry instanceof $this->objectType) {
            return $entry;
        }
        
        $dataMapper = GeneralUtility::makeInstance(DataMapper::class);
        $tablename = $dataMapper->getDataMap($this->objectType)->getTableName();
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($tablename);
        //SELECT * FROM tx_iggeodata_domain_model_plz ORDER BY abs(8000-plz) LIMIT 1;

        // sinnvoll? Bei z.B. 8000 sollen 7xxx nicht berücksichtigt werden, da wohl weiter weg (findet aber eh die ueber 8000)
        // only search for postcodes with the same first and second digits
        $firstDigits = intval($plz/100);
        $result = $queryBuilder->select('*')->from($tablename)->where(
            $queryBuilder->expr()->andX(
                $queryBuilder->expr()->gte('plz', $firstDigits * 100),
                $queryBuilder->expr()->lt('plz', ($firstDigits + 1) * 100),
            ),
        )->add('orderBy', 'abs(' . (int)$plz . '-plz)')->setMaxResults(1)->executeQuery();
        $row = $result->fetchAssociative();
        if ($row === false) {
            return $row;
        }
        $entry =  $dataMapper->map($this->objectType, [$row])[0];
        return $entry ;                
    }    
}
